' Add a reference to System.DirectoryServices
' and System.Management
Imports System.DirectoryServices
Imports System.Management
Imports System.Configuration.ConfigurationSettings
Imports System.Collections.Specialized
Imports System.IO

Module Main

    Sub Main()
        Dim Server, WinDir, JobFile As String
        Dim ServerList As StringCollection
        Dim wmi As ManagementClass
        Dim obj As ManagementObject
        Dim sc As New ScheduledTaskByFile
        Dim UNC, msg, buf, Names(), JobName As String
        Dim st As New SQLTasks
        Dim el As New EventLog

        Console.WriteLine("TaskAlert")
        Console.WriteLine("Start: " & Now)


        ' Get a list of Servers
        ServerList = ListServers(My.Settings.Domain, My.Settings.OU)
        If IsNothing(ServerList) Then
            Console.WriteLine("Yikes, can't get the list of servers")
            System.Threading.Thread.Sleep(5000)

            el.Source = "TaskAlert"
            el.WriteEntry("Yikes, can't get the list of servers", EventLogEntryType.Error)
            Environment.ExitCode = 1
            Exit Sub
        End If

        ' Any machines to explicitly add?
        If Not My.Settings.IncludeList Is Nothing Then
            For Each Server In My.Settings.IncludeList
                If Not ServerList.Contains(Server) Then
                    ServerList.Add(Server)
                End If
            Next
        End If

        ' Any machine to explicity remove?
        If Not My.Settings.ExcludeList Is Nothing Then
            For Each Server In My.Settings.ExcludeList
                If ServerList.Contains(Server) Then
                    ServerList.Remove(Server)
                End If
            Next
        End If

        ' loop thru each server
        msg = ""
        For Each Server In ServerList
            Console.Write(".")

            Try
                ' Get the location/name of the Windows directory
                WinDir = ""
                wmi = New ManagementClass("\\" & Server & "\root\cimv2:Win32_OperatingSystem")
                For Each obj In wmi.GetInstances()
                    WinDir = obj("WindowsDirectory").ToString
                Next
                wmi.Dispose()

                ' check the status of the Windows Scheduled Tasks
                UNC = ConvUNC(Server, WinDir & "\tasks")
                For Each JobFile In Directory.GetFiles(UNC, "*.job")
                    buf = sc.GetTaskInfo(JobFile)
                    If buf <> "" Then
                        Names = JobFile.Split("\"c)
                        JobName = Names(UBound(Names)).Replace(".job", "")
                        msg &= "Job '" & JobName & "' on " & Server & " " & buf & vbCr
                    End If
                Next

                ' check the status of SQL Agent Jobs
                buf = st.CheckTasks(Server)
                If buf <> "" Then
                    msg &= buf
                End If
            Catch ex As Exception
                Console.WriteLine(vbCr & "Yikes, Something went wrong processing server " & Server)
                System.Threading.Thread.Sleep(5000)

                el.Source = "TaskAlert"
                el.WriteEntry("Internal Error processing server " & Server & vbCr & ex.Message & vbCr & ex.StackTrace, EventLogEntryType.Warning)
                ' non-fatal
            End Try
        Next

        Console.WriteLine("")
        If msg <> "" Then

            msg = "The following Scheduled Tasks have failed:" & vbCr & vbCr & msg
            Console.WriteLine(msg)

            Try
                Dim a As New Alerts
                a.SendMessage(msg)
                a.SendEventLog(msg)
                a.SendEmail(msg)
            Catch ex As Exception
                Environment.ExitCode = Err.Number
                Console.WriteLine("Can't perform one of the alert actions")
                System.Threading.Thread.Sleep(5000)
                Dim ErrMsg As String = ""
                Do While Not IsNothing(ex)
                    ErrMsg &= ex.Message & vbCr
                    ex = ex.InnerException
                Loop
                el.Source = "TaskAlert"
                el.WriteEntry("Can't perform one of the alert actions" & vbCr & ErrMsg, EventLogEntryType.Error)
                Exit Sub
            End Try
        Else
            el.Source = "TaskAlert"
            el.WriteEntry("Completed Successfully", EventLogEntryType.Information)
        End If

        Console.WriteLine("Ended: " & Now)
        System.Threading.Thread.Sleep(5000)
    End Sub

    Function ListServers(ByVal DomainName As String, ByVal StartingOU As String) As StringCollection
        Dim entry, de As DirectoryEntry
        Dim srch As DirectorySearcher
        Dim result As SearchResult
        Dim ldap As String
        Dim ans As New StringCollection
        Dim i As Integer

        ' Let's find the full path to the OU
        ldap = "LDAP://" & DomainName & "/" & StartingOU

        entry = New DirectoryEntry(ldap)
        srch = New DirectoryServices.DirectorySearcher(entry)

        ' an LDAP filter for servers
        srch.Filter = "(&(ObjectCategory=computer)(operatingSystem=*Server*))"

        ' OK, let's do it
        Try
            For Each result In srch.FindAll()
                de = result.GetDirectoryEntry
                ans.Add(de.Properties("CN").Value.ToString)
                i += 1
            Next
        Catch ex As Exception
            Return Nothing
        End Try

        Return ans
    End Function

    ' Convert a "local" filename to a UNC remote filename
    Private Function ConvUNC(ByVal RemotePC As String, ByVal Filename As String) As String
        Dim ans As String = Nothing

        If RemotePC = "" Then
            Return Filename
        End If

        If Len(Filename) > 0 Then
            ans = "\\" & RemotePC & "\" & Filename.Substring(0, 1) & "$" & Filename.Substring(2)
        End If

        Return ans
    End Function
End Module

